package it.turboApiClient.client;

import static org.junit.Assert.*;

import java.io.IOException;

import it.turboApiClient.model.Mail;

import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import com.google.gson.Gson;

public class TurboApiClientTest {
	
    private static final String USERNAME                  = "username@turbo-smtp.com"; //insert TurboSMTP Account "username"
    private static final String PASSWORD                  = "inserpasswordhere"; //insert TurboSMTP Account "password"
	private static final String SERVER_URL         	      = "https://api.turbo-smtp.com/api/v2/mail/send";
	private static final String INCORRECT_SERVER_URL      = "https://mock-url.morg";
	private static final String FROM_MAIL_FIELD           = "from@email.com";  //insert email "from" value
	private static final String INCORRECT_FROM_MAIL_FIELD = "an_incorrect_@email_address_from";
	private static final String TO_MAIL_FIELD             = "to@password.com"; //insert email "to" value
	private static final String INCORRECT_TO_MAIL_FIELD   = "an_incorrect_@email_address_to";
	private static final String SUBJECT_CONTENT           = "Java Test TurboAPI Send(email) : campo SUBJECT";
	private static final String HTML_CONTENT              = "Java Test TurboAPI Send(email) : campo HTML_CONTENT";
	   
	@Rule
	public ExpectedException exception = ExpectedException.none();

	
	@Test
	public void test_messageOK() {
		Mail mail = new Mail();
        mail.setFrom(FROM_MAIL_FIELD);
        mail.setTo(TO_MAIL_FIELD);
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT);
        
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
		
		String returnedMessage;
		try {
			returnedMessage = (new TurboApiClient()).sendMail(USERNAME, PASSWORD, SERVER_URL, jsonMailString);
			if ("{\"message\":\"OK\"}".equals(returnedMessage)) {
	        	assertTrue(true);
	        } else if ("{\"message\":\"error\",\"errors\":{\"message\":\"error\",\"errors\":\"nocredit\"}}".equals(returnedMessage)) {
	        	assertTrue(true);
	        } else {
	        	fail("Unknown returned message!!");
	        }    
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}        
	}
	
	//case: "from" parameter is missing  -->The mail is not send
	@Test
	public void test_message_From_Param_Not_Found() {
		Mail mail = new Mail();
        mail.setTo(TO_MAIL_FIELD);
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT); 
        
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
		
		String returnedMessage;
		try {
			returnedMessage = (new TurboApiClient()).sendMail(USERNAME, PASSWORD, SERVER_URL, jsonMailString);
			assertEquals("{\"message\":\"error\",\"errors\":[\"missing or not valid sender email (from)\"]}", returnedMessage); 
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
          
	}
	
	//case: "from" parameter is incorrect  -->The mail is not send
	@Test
	public void test_message_From_Param_Incorrect() {
		Mail mail = new Mail();
        mail.setFrom(INCORRECT_TO_MAIL_FIELD);
        mail.setTo(TO_MAIL_FIELD);
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT); 
     
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
		
		String returnedMessage;
		try {
			returnedMessage = (new TurboApiClient()).sendMail(USERNAME, PASSWORD, SERVER_URL, jsonMailString);
			assertEquals("{\"message\":\"error\",\"errors\":[\"missing or not valid sender email (from)\"]}", returnedMessage); 
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	//case: "to" parameter is missing  -->The mail is not send
	@Test
	public void test_message_To_Param_Not_Found() {
		Mail mail = new Mail();
        mail.setFrom(FROM_MAIL_FIELD);
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT); 
        
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
		
		String returnedMessage;
		try {
			returnedMessage = (new TurboApiClient()).sendMail(USERNAME, PASSWORD, SERVER_URL, jsonMailString);
			assertEquals("{\"message\":\"error\",\"errors\":[\"missing recipients (to)\"]}", returnedMessage); 
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}   
	}
       
	//case: "to" parameter is incorrect  -->The mail is not send
	@Test
	public void test_message_To_Param_Incorrect() {
		Mail mail = new Mail();
        mail.setFrom(FROM_MAIL_FIELD);
        mail.setTo(INCORRECT_TO_MAIL_FIELD);
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT); 
        
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
		
		String returnedMessage;
		try {
			returnedMessage = (new TurboApiClient()).sendMail(USERNAME, PASSWORD, SERVER_URL, jsonMailString);
			assertEquals("{\"message\":\"error\",\"errors\":[[\"\'" + INCORRECT_TO_MAIL_FIELD + "\' \'to\' email not valid\"]]}", returnedMessage); 
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}      
	}
		
	
	//case: "from" and "to" parameter are missing  -->The mail is not send
	@Test
	public void test_message_From_And_To_Param_Not_Found() {
		Mail mail = new Mail();
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT); 
        
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
		
		String returnedMessage;
		try {
			returnedMessage = (new TurboApiClient()).sendMail(USERNAME, PASSWORD, SERVER_URL, jsonMailString);
			assertEquals("{\"message\":\"error\",\"errors\":[\"missing or not valid sender email (from)\",\"missing recipients (to)\"]}", returnedMessage); 
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}      
		
	}
	
	//case: "from" and "to" parameter are incorrect  -->The mail is not send
	@Test
	public void test_message_From_And_To_Param_Incorrect() {
		Mail mail = new Mail();
        mail.setFrom(INCORRECT_FROM_MAIL_FIELD);
        mail.setTo(INCORRECT_TO_MAIL_FIELD);
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT); 
        
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
		
		String returnedMessage;
		try {
			returnedMessage = (new TurboApiClient()).sendMail(USERNAME, PASSWORD, SERVER_URL, jsonMailString);
			assertEquals("{\"message\":\"error\",\"errors\":[\"missing or not valid sender email (from)\",[\"\'" + INCORRECT_TO_MAIL_FIELD + "\' \'to\' email not valid\"]]}", returnedMessage);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}      
		 
	}
	
	//case: incorrect authentication credentials (username)
	@Test (expected=IOException.class)
	public void  test_error412_wrong_username() throws Exception {
		Mail mail = new Mail();
        mail.setFrom(FROM_MAIL_FIELD);
        mail.setTo(TO_MAIL_FIELD);
        mail.setSubject(SUBJECT_CONTENT);
        mail.setHtml_content(HTML_CONTENT);
        
        Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
	
		try {
			(new TurboApiClient()).sendMail("", PASSWORD, SERVER_URL, jsonMailString);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw e;
		}     
	}
	
	//case: incorrect authentication credentials(password)
	@Test (expected=IOException.class)
	public void test_error412_wrong_password() throws Exception {
		Mail mail = new Mail();
	    mail.setFrom(FROM_MAIL_FIELD);
	    mail.setTo(TO_MAIL_FIELD);
	    mail.setSubject(SUBJECT_CONTENT);
	    mail.setHtml_content(HTML_CONTENT);
		
	    Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
	
	    try {
			(new TurboApiClient()).sendMail(USERNAME, "", SERVER_URL, jsonMailString);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			throw e;
		}  
	}
	  
	//case: incorrect resource URL or resource not found
	@Test (expected=IOException.class)
	public void test_error404() throws Exception {
		Mail mail = new Mail();
	    mail.setFrom(FROM_MAIL_FIELD);
	    mail.setTo(TO_MAIL_FIELD);
	    mail.setSubject(SUBJECT_CONTENT);
	    mail.setHtml_content(HTML_CONTENT);
		
	    Gson jsonConverter = new Gson();
		String jsonMailString = jsonConverter.toJson(mail);
	
	    try {
			(new TurboApiClient()).sendMail(USERNAME, PASSWORD, INCORRECT_SERVER_URL, jsonMailString);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw e;
		}
	}


        

}
